from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class ResourceSelectionDefinition(JsonModel):
    """Selection metadata associated with the resource."""

    id: str | None = None
    """The ID of the resource reserved for the work item."""

    target_location_id: str | None = None
    """The ID of the location where the resource is to be moved."""

    target_system_id: str | None = None
    """The ID of the system where the resource is to be moved."""

    target_parent_id: str | None = None
    """The ID of the parent to which the resource is to be connected."""


class SystemResourceSelectionDefinition(JsonModel):
    """Selection metadata associated with the system resource."""

    id: str | None = None
    """The ID of the system reserved for the work item."""

    target_location_id: str | None = None
    """The ID of the location where the system is to be moved."""


class ResourceDefinition(JsonModel):
    """Resource reserved for the work item."""

    selections: List[ResourceSelectionDefinition] | None = None
    """Resource selections for the work item."""

    filter: str | None = None
    """The filter used to select the resources for the work item."""


class SystemResourceDefinition(JsonModel):
    """System resource reserved for the work item."""

    selections: List[SystemResourceSelectionDefinition] | None = None
    """System resource selections for the work item."""

    filter: str | None = None
    """The filter used to select the systems for the work item."""


class ResourcesDefinition(JsonModel):
    """Resources reserved for the work item."""

    assets: ResourceDefinition | None = None
    """Asset reservations for the work item."""

    duts: ResourceDefinition | None = None
    """DUT reservations for the work item."""

    fixtures: ResourceDefinition | None = None
    """Fixture reservations for the work item."""

    systems: SystemResourceDefinition | None = None
    """System reservations for the work item."""


class TemplateResourceDefinition(JsonModel):
    """Resource reserved for the work item created from a template."""

    filter: str | None = None
    """The filter used to select the resources for the work item."""


class TemplateResourcesDefinition(JsonModel):
    """Resources reserved for the work item created from a template."""

    assets: TemplateResourceDefinition | None = None
    """Asset reservations for the work item."""

    duts: TemplateResourceDefinition | None = None
    """DUT reservations for the work item."""

    fixtures: TemplateResourceDefinition | None = None
    """Fixture reservations for the work item."""

    systems: TemplateResourceDefinition | None = None
    """System reservations for the work item."""


class ScheduleResourceDefinition(JsonModel):
    """Resource reserved for scheduling the work item."""

    selections: List[ResourceSelectionDefinition] | None = None
    """Resource selections for the work item."""


class ScheduleSystemResourceDefinition(JsonModel):
    """System resource reserved for scheduling the work item."""

    selections: List[SystemResourceSelectionDefinition] | None = None
    """System resource selections for the work item."""


class ScheduleResourcesDefinition(JsonModel):
    """Resources reserved for scheduling the work item."""

    assets: ScheduleResourceDefinition | None = None
    """Asset reservations for the work item."""

    duts: ScheduleResourceDefinition | None = None
    """DUT reservations for the work item."""

    fixtures: ScheduleResourceDefinition | None = None
    """Fixture reservations for the work item."""

    systems: ScheduleSystemResourceDefinition | None = None
    """System reservations for the work item."""



from typing import List

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._work_item import WorkItem


class PagedWorkItems(WithPaging):
    """The response containing the list of work items, total count and continuation token."""

    work_items: List[WorkItem]
    """A list of all the work items in this page."""

    total_count: int | None = None
    """The total number of work items that match the query."""



from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._resources_definition import ScheduleResourcesDefinition
from ._schedule_definition import ScheduleDefinition


class ScheduleWorkItemRequest(JsonModel):
    """Represents the request body content for scheduling a single work item."""

    id: str
    """The ID of the work item to be scheduled."""

    assigned_to: str | None = None
    """The ID of the user to whom the work item is assigned."""

    schedule: ScheduleDefinition | None = None
    """Scheduling properties for the work item."""

    resources: ScheduleResourcesDefinition | None = None
    """Resources reserved for scheduling the work item."""



from ._work_item import WorkItem
from ._work_item_template import WorkItemTemplate, WorkItemTemplateBase

from ._state import State
from ._query_work_items_request import WorkItemField, WorkItemOrderBy
from ._query_work_item_templates_request import (
    WorkItemTemplateField,
    WorkItemTemplateOrderBy,
)
from ._dashboard import Dashboard, DashboardUrl
from ._resources_definition import (
    ResourceDefinition,
    ResourceSelectionDefinition,
    ResourcesDefinition,
    ScheduleResourceDefinition,
    ScheduleResourcesDefinition,
    ScheduleSystemResourceDefinition,
    SystemResourceDefinition,
    SystemResourceSelectionDefinition,
    TemplateResourceDefinition,
    TemplateResourcesDefinition,
)
from ._schedule_definition import ScheduleDefinition
from ._timeline_definition import TemplateTimelineDefinition, TimelineDefinition
from ._execution_definition import (
    ExecutionDefinition,
    Job,
    JobExecution,
    ManualExecution,
    NoneExecution,
    NotebookExecution,
)
from ._execution_event import (
    ExecutionEvent,
    ExecutionEventBase,
    JobExecutionEvent,
    ManualExecutionEvent,
    NotebookExecutionEvent,
)

from ._create_work_item_request import CreateWorkItemRequest
from ._create_work_items_partial_success_response import (
    CreateWorkItemsPartialSuccessResponse,
)
from ._delete_work_items_partial_success_response import (
    DeleteWorkItemsPartialSuccessResponse,
)
from ._paged_work_items import PagedWorkItems
from ._query_work_items_request import QueryWorkItemsRequest
from ._schedule_work_item_request import ScheduleWorkItemRequest
from ._schedule_work_items_partial_success_response import (
    ScheduleWorkItemsPartialSuccessResponse,
)
from ._schedule_work_items_request import ScheduleWorkItemsRequest
from ._update_work_item_request import UpdateWorkItemRequest
from ._update_work_items_partial_success_response import (
    UpdateWorkItemsPartialSuccessResponse,
)
from ._update_work_items_request import UpdateWorkItemsRequest

from ._create_work_item_template_request import CreateWorkItemTemplateRequest
from ._create_work_item_templates_partial_success_response import (
    CreateWorkItemTemplatesPartialSuccessResponse,
)
from ._delete_work_item_templates_partial_success_response import (
    DeleteWorkItemTemplatesPartialSuccessResponse,
)
from ._paged_work_item_templates import PagedWorkItemTemplates
from ._query_work_item_templates_request import QueryWorkItemTemplatesRequest
from ._update_work_item_template_request import UpdateWorkItemTemplateRequest
from ._update_work_item_templates_request import UpdateWorkItemTemplatesRequest
from ._update_work_item_templates_partial_success_response import (
    UpdateWorkItemTemplatesPartialSuccessResponse,
)

# flake8: noqa



from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._update_work_item_request import UpdateWorkItemRequest
from ._work_item import WorkItem


class UpdateWorkItemsPartialSuccessResponse(JsonModel):
    """Response for updating work items with partial success."""

    updated_work_items: List[WorkItem] | None = None
    """List of successfully updated work items."""

    failed_work_items: List[UpdateWorkItemRequest] | None = None
    """List of work item requests that failed to update."""

    error: ApiError | None = None
    """The error that occurred when updating the work items."""



from enum import Enum
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class WorkItemField(str, Enum):
    """Enumeration of work item fields that can be projected in query results."""

    ID = "ID"
    NAME = "NAME"
    TYPE = "TYPE"
    STATE = "STATE"
    SUBSTATE = "SUBSTATE"
    DESCRIPTION = "DESCRIPTION"
    PARENT_ID = "PARENT_ID"
    TEMPLATE_ID = "TEMPLATE_ID"
    ASSIGNED_TO = "ASSIGNED_TO"
    REQUESTED_BY = "REQUESTED_BY"
    TEST_PROGRAM = "TEST_PROGRAM"
    PART_NUMBER = "PART_NUMBER"
    TIMELINE = "TIMELINE"
    TIMELINE_EARLIEST_START_DATE_TIME = "TIMELINE_EARLIEST_START_DATE_TIME"
    TIMELINE_DUE_DATE_TIME = "TIMELINE_DUE_DATE_TIME"
    TIMELINE_ESTIMATED_DURATION_IN_SECONDS = "TIMELINE_ESTIMATED_DURATION_IN_SECONDS"
    SCHEDULE = "SCHEDULE"
    SCHEDULE_PLANNED_START_DATE_TIME = "SCHEDULE_PLANNED_START_DATE_TIME"
    SCHEDULE_PLANNED_END_DATE_TIME = "SCHEDULE_PLANNED_END_DATE_TIME"
    SCHEDULE_PLANNED_DURATION_IN_SECONDS = "SCHEDULE_PLANNED_DURATION_IN_SECONDS"
    RESOURCES = "RESOURCES"
    RESOURCES_SYSTEMS = "RESOURCES_SYSTEMS"
    RESOURCES_ASSETS = "RESOURCES_ASSETS"
    RESOURCES_DUTS = "RESOURCES_DUTS"
    RESOURCES_FIXTURES = "RESOURCES_FIXTURES"
    RESOURCES_SYSTEMS_SELECTIONS = "RESOURCES_SYSTEMS_SELECTIONS"
    RESOURCES_SYSTEMS_SELECTIONS_ID = "RESOURCES_SYSTEMS_SELECTIONS_ID"
    RESOURCES_SYSTEMS_SELECTIONS_TARGET_LOCATION_ID = (
        "RESOURCES_SYSTEMS_SELECTIONS_TARGET_LOCATION_ID"
    )
    RESOURCES_ASSETS_SELECTIONS = "RESOURCES_ASSETS_SELECTIONS"
    RESOURCES_ASSETS_SELECTIONS_ID = "RESOURCES_ASSETS_SELECTIONS_ID"
    RESOURCES_ASSETS_SELECTIONS_TARGET_LOCATION_ID = (
        "RESOURCES_ASSETS_SELECTIONS_TARGET_LOCATION_ID"
    )
    RESOURCES_ASSETS_SELECTIONS_TARGET_SYSTEM_ID = (
        "RESOURCES_ASSETS_SELECTIONS_TARGET_SYSTEM_ID"
    )
    RESOURCES_ASSETS_SELECTIONS_TARGET_PARENT_ID = (
        "RESOURCES_ASSETS_SELECTIONS_TARGET_PARENT_ID"
    )
    RESOURCES_DUTS_SELECTIONS = "RESOURCES_DUTS_SELECTIONS"
    RESOURCES_DUTS_SELECTIONS_ID = "RESOURCES_DUTS_SELECTIONS_ID"
    RESOURCES_DUTS_SELECTIONS_TARGET_LOCATION_ID = (
        "RESOURCES_DUTS_SELECTIONS_TARGET_LOCATION_ID"
    )
    RESOURCES_DUTS_SELECTIONS_TARGET_PARENT_ID = (
        "RESOURCES_DUTS_SELECTIONS_TARGET_PARENT_ID"
    )
    RESOURCES_DUTS_SELECTIONS_TARGET_SYSTEM_ID = (
        "RESOURCES_DUTS_SELECTIONS_TARGET_SYSTEM_ID"
    )
    RESOURCES_FIXTURES_SELECTIONS = "RESOURCES_FIXTURES_SELECTIONS"
    RESOURCES_FIXTURES_SELECTIONS_ID = "RESOURCES_FIXTURES_SELECTIONS_ID"
    RESOURCES_FIXTURES_SELECTIONS_TARGET_LOCATION_ID = (
        "RESOURCES_FIXTURES_SELECTIONS_TARGET_LOCATION_ID"
    )
    RESOURCES_FIXTURES_SELECTIONS_TARGET_PARENT_ID = (
        "RESOURCES_FIXTURES_SELECTIONS_TARGET_PARENT_ID"
    )
    RESOURCES_FIXTURES_SELECTIONS_TARGET_SYSTEM_ID = (
        "RESOURCES_FIXTURES_SELECTIONS_TARGET_SYSTEM_ID"
    )
    RESOURCES_ASSETS_FILTER = "RESOURCES_ASSETS_FILTER"
    RESOURCES_DUTS_FILTER = "RESOURCES_DUTS_FILTER"
    RESOURCES_FIXTURES_FILTER = "RESOURCES_FIXTURES_FILTER"
    RESOURCES_SYSTEMS_FILTER = "RESOURCES_SYSTEMS_FILTER"
    FILE_IDS_FROM_TEMPLATE = "FILE_IDS_FROM_TEMPLATE"
    PROPERTIES = "PROPERTIES"
    DASHBOARD = "DASHBOARD"
    CREATED_BY = "CREATED_BY"
    CREATED_AT = "CREATED_AT"
    UPDATED_BY = "UPDATED_BY"
    UPDATED_AT = "UPDATED_AT"
    WORKSPACE = "WORKSPACE"
    WORKFLOW_ID = "WORKFLOW_ID"


class WorkItemOrderBy(str, Enum):
    """Enumeration of fields by which work items can be ordered."""

    ID = "ID"
    UPDATED_AT = "UPDATED_AT"


class QueryWorkItemsRequest(JsonModel):
    """Represents the request body content for querying work items.
    Allows filtering, sorting, and pagination of work item results.
    """

    filter: str | None = None
    """A string expression to filter the work items returned by the query.

    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable.
    """

    substitutions: List[bool | int | str | None] | None = None
    """Makes substitutions in the query filter expression.

    Substitutions for the query expression are indicated by non-negative integers that are
    prefixed with the @ symbol. Each substitution in the given expression will be replaced by
    the element at the corresponding index (zero-based) in this list.
    """

    take: int | None = None
    """The maximum number of work items to return in the response."""

    continuation_token: str | None = None
    """A token which allows the user to resume a query at the next item in the matching results.

    When querying, a token will be returned if a query may be continued. To obtain the next page
    of results, pass the token to the service on a subsequent request.
    """

    order_by: WorkItemOrderBy | None = None
    """The field name to use for sorting the work items."""

    descending: bool | None = None
    """Whether to sort the work items in descending order."""

    return_count: bool | None = None
    """Whether to include the total count of matching work items in the response."""

    projection: List[WorkItemField] | None = None
    """
    Gets or sets the projection to be used when retrieving the work items. If not specified,
    all properties will be returned.
    """



from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class DeleteWorkItemsPartialSuccessResponse(JsonModel):
    """Response for deleting work items with partial success."""

    deleted_ids: List[str] | None = None
    """List of work item IDs that were successfully deleted."""

    failed_ids: List[str] | None = None
    """List of work item IDs that failed to delete."""

    error: ApiError | None = None
    """The error that occurred when deleting the work items."""



from typing import List

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._work_item_template import WorkItemTemplate


class PagedWorkItemTemplates(WithPaging):
    """The response containing the list of work item templates, total count and continuation token."""

    work_item_templates: List[WorkItemTemplate]
    """A list of all the work item templates in this page."""

    total_count: int | None = None
    """The total number of work item templates that match the query."""



from nisystemlink.clients.core._uplink._json_model import JsonModel


class BaseFileRequest(JsonModel):
    """Base class for file request models containing common query parameters."""

    skip: int | None = None
    """
    How many files to skip in the result when paging.
    """

    take: int | None = None
    """
    How many files to return in the result.
    """

    order_by_descending: bool | None = False
    """
    Whether to sort in descending order.
    """



from nisystemlink.clients.file.models._base_file_request import BaseFileRequest
from nisystemlink.clients.file.models._file_query_order_by import SearchFilesOrderBy


class SearchFilesRequest(BaseFileRequest):
    """Request model for searching files."""

    filter: str | None = None
    """
    The filter criteria for files using Lucene query syntax. Consists of a string of queries composed
    using AND/OR operators. String values and date strings need to be enclosed in double quotes.
    Parentheses can be used around filters to better define the order of operations.
    Example Filter syntax: '[property name]:[operand] AND [property name]:[operand]'
    """

    order_by: SearchFilesOrderBy | None = None
    """
    The property by which to order the files in the response.
    """



from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import Dashboard
from ._execution_definition import ExecutionDefinition
from ._resources_definition import ResourcesDefinition
from ._timeline_definition import TimelineDefinition


class CreateWorkItemRequest(JsonModel):
    """Represents the request body content for creating a work item."""

    name: str | None = None
    """The name of the work item."""

    type: str | None = None
    """The type of the work item."""

    state: str | None = None
    """The state of the work item."""

    description: str | None = None
    """A description of the work item."""

    parent_id: str | None = None
    """The ID of the parent work item."""

    template_id: str | None = None
    """The ID of the template used to create the work item."""

    assigned_to: str | None = None
    """The user or group assigned to the work item."""

    requested_by: str | None = None
    """The user or group who requested the work item."""

    test_program: str | None = None
    """The test program associated with the work item."""

    part_number: str | None = None
    """The part number associated with the work item."""

    workspace: str | None = None
    """The workspace to which the work item belongs."""

    timeline: TimelineDefinition | None = None
    """Timeline properties for the work item."""

    resources: ResourcesDefinition | None = None
    """Resources reserved for the work item."""

    execution_actions: List[ExecutionDefinition] | None = None
    """The execution actions defined for the work item."""

    file_ids_from_template: List[str] | None = None
    """The list of file IDs inherited from the template."""

    properties: Dict[str, str] | None = None
    """Additional properties for the work item."""

    dashboard: Dashboard | None = None
    """The dashboard associated with the work item."""

    workflow_id: str | None = None
    """The ID of the workflow associated with the work item."""



from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._schedule_work_item_request import ScheduleWorkItemRequest


class ScheduleWorkItemsRequest(JsonModel):
    """Represents the request body content for scheduling multiple work items."""

    work_items: List[ScheduleWorkItemRequest]
    """List of work items to be scheduled."""

    replace: bool | None = None
    """When true, existing array fields are replaced instead of merged."""



from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._update_work_item_template_request import UpdateWorkItemTemplateRequest
from ._work_item_template import WorkItemTemplate


class UpdateWorkItemTemplatesPartialSuccessResponse(JsonModel):
    """Response for updating work item templates with partial success."""

    updated_work_item_templates: List[WorkItemTemplate]
    """List of successfully updated work item templates."""

    failed_work_item_templates: List[UpdateWorkItemTemplateRequest] | None = None
    """List of work item template requests that failed to update."""

    error: ApiError | None = None
    """The error that occurred when updating the work item templates."""



from ._file_metadata import FileMetadata
from ._file_query_order_by import (
    FileQueryOrderBy,
    FileLinqQueryOrderBy,
    SearchFilesOrderBy,
)
from ._file_query_response import FileQueryResponse
from ._link import Link
from ._operations import V1Operations
from ._update_metadata import UpdateMetadataRequest
from ._file_linq_query import FileLinqQueryRequest, FileLinqQueryResponse
from ._search_files_request import SearchFilesRequest
from ._search_files_response import SearchFilesResponse
from ._base_file_response import BaseFileResponse, TotalCount, TotalCountRelation
from ._base_file_request import BaseFileRequest

# flake8: noqa



from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import DashboardUrl
from ._execution_definition import ExecutionDefinition
from ._resources_definition import TemplateResourcesDefinition
from ._timeline_definition import TemplateTimelineDefinition


class UpdateWorkItemTemplateRequest(JsonModel):
    """Represents the request body content for updating a single work item template."""

    id: str
    """The ID of the work item template."""

    product_families: List[str] | None = None
    """The array of product families to which the work item template belongs."""

    part_numbers: List[str] | None = None
    """The array of part numbers of the products linked to the work item template."""

    name: str | None = None
    """The name of the work item template."""

    summary: str | None = None
    """The summary of the work item template."""

    description: str | None = None
    """The description of the work item created from this template."""

    template_group: str | None = None
    """The template group defined by the user."""

    test_program: str | None = None
    """The test program associated with the work item created from this template."""

    timeline: TemplateTimelineDefinition | None = None
    """Timeline properties for the work item created from this template."""

    resources: TemplateResourcesDefinition | None = None
    """Resources selection criteria for the work item created from this template."""

    execution_actions: List[ExecutionDefinition] | None = None
    """The execution actions defined for the work item."""

    file_ids: List[str] | None = None
    """The array of file IDs associated with the work item template."""

    workspace: str | None = None
    """The workspace to which the work item template belongs."""

    properties: Dict[str, str] | None = None
    """Additional properties associated with the work item created from this template."""

    dashboard: DashboardUrl | None = None
    """The dashboard data related to the work item created from this template."""

    workflow_id: str | None = None
    """The ID of the workflow associated with the work item created from this template."""



from datetime import datetime
from typing import Annotated, List, Literal

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import Field

from ._state import State


class ExecutionEventBase(JsonModel):
    """Base class for execution events containing common attributes."""

    action: str | None = None
    """The action the execution was triggered for."""

    triggered_at: datetime | None = None
    """The time the execution was triggered."""

    triggered_by: str | None = None
    """The user who triggered the execution."""

    previous_state: State | None = None
    """The state of the work item at the time of execution."""

    previous_substate: str | None = None
    """The substate of the work item at the time of execution."""

    new_state: State | None = None
    """The state of the work item resulting from execution."""

    new_substate: str | None = None
    """The substate of the work item resulting from execution."""

    error: ApiError | None = None
    """The error from the execution, if any."""


class NotebookExecutionEvent(ExecutionEventBase):
    """An event tracking a notebook execution triggered from a work item."""

    type: Literal["NOTEBOOK"] = Field(default="NOTEBOOK")
    """Type of execution, default is 'NOTEBOOK'."""

    execution_id: str | None = None
    """The ID of the triggered execution."""


class JobExecutionEvent(ExecutionEventBase):
    """An event tracking a job execution triggered from a work item."""

    type: Literal["JOB"] = Field(default="JOB")
    """Type of execution, default is 'JOB'."""

    job_ids: List[str] | None = None
    """The list of job IDs."""


class ManualExecutionEvent(ExecutionEventBase):
    """An event tracking a manual execution triggered from a work item."""

    type: Literal["MANUAL"] = Field(default="MANUAL")
    """Type of execution, default is 'MANUAL'."""


ExecutionEvent = Annotated[
    NotebookExecutionEvent | ManualExecutionEvent | JobExecutionEvent,
    Field(discriminator="type"),
]



from enum import Enum
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._file_metadata import LinqQueryFileMetadata


class TotalCountRelation(str, Enum):
    """Describes the relation the returned total count value has with respect to the total number of files."""

    EQUALS = "eq"
    """Equals, meaning that the returned items are all the items that matched the filter."""

    GREATER_THAN_OR_EQUAL = "gte"
    """Greater or equal, meaning that the take limit has been hit, but there are further items that match the query."""


class TotalCount(JsonModel):
    """The total number of files that match the query regardless of skip and take values"""

    relation: TotalCountRelation
    """
    Describes the relation the returned total count value has with respect to the total number of
    files matched by the query.
    """

    value: int
    """Describes the number of files that were returned as a result of the query in the database"""


class BaseFileResponse(JsonModel):
    """Base class for file response models containing a list of files and total count."""

    available_files: List[LinqQueryFileMetadata]
    """The list of files returned by the query"""

    total_count: TotalCount
    """The total number of files that match the query regardless of skip and take values"""



from datetime import datetime
from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import DashboardUrl
from ._execution_definition import ExecutionDefinition
from ._execution_event import ExecutionEvent
from ._resources_definition import ResourcesDefinition
from ._schedule_definition import ScheduleDefinition
from ._state import State
from ._timeline_definition import TimelineDefinition


class WorkItem(JsonModel):
    """Contains information about a work item."""

    id: str | None = None
    """The ID of the work item."""

    template_id: str | None = None
    """The ID of the template used to create the work item."""

    name: str | None = None
    """The name of the work item."""

    type: str | None = None
    """The type of the work item."""

    state: State | None = None
    """The state of the work item."""

    substate: str | None = None
    """The substate of the work item, if any."""

    description: str | None = None
    """The description of the work item."""

    parent_id: str | None = None
    """The ID of the parent work item."""

    assigned_to: str | None = None
    """The user or group assigned to the work item."""

    requested_by: str | None = None
    """The user or group who requested the work item."""

    test_program: str | None = None
    """The test program associated with the work item."""

    part_number: str | None = None
    """The part number associated with the work item."""

    workspace: str | None = None
    """The workspace to which the work item belongs."""

    timeline: TimelineDefinition | None = None
    """Timeline properties for the work item."""

    schedule: ScheduleDefinition | None = None
    """Scheduling properties for the work item."""

    resources: ResourcesDefinition | None = None
    """Resources reserved for the work item."""

    created_by: str | None = None
    """The user who created the work item."""

    updated_by: str | None = None
    """The user who last updated the work item."""

    created_at: datetime | None = None
    """The date and time when the work item was created."""

    updated_at: datetime | None = None
    """The date and time when the work item was last updated."""

    properties: Dict[str, str] | None = None
    """Additional properties associated with the work item."""

    file_ids_from_template: List[str] | None = None
    """The list of file IDs inherited from the template."""

    dashboard: DashboardUrl | None = None
    """The dashboard data related to the work item."""

    execution_actions: List[ExecutionDefinition] | None = None
    """The execution actions defined for the work item."""

    execution_history: List[ExecutionEvent] | None = None
    """The execution history of the work item."""

    workflow_id: str | None = None
    """The ID of the workflow associated with the work item."""



from datetime import datetime

from nisystemlink.clients.core._uplink._json_model import JsonModel


class TimelineDefinition(JsonModel):
    """Timeline properties for the work item."""

    earliest_start_date_time: datetime | None = None
    """The earliest start date and time for the work item."""

    due_date_time: datetime | None = None
    """The due date and time for the work item."""

    estimated_duration_in_seconds: int | None = None
    """The estimated duration of the work item in seconds."""


class TemplateTimelineDefinition(JsonModel):
    """Timeline properties for the work item created from a template."""

    estimated_duration_in_seconds: int | None = None
    """The estimated duration of the work item in seconds."""



from enum import Enum
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class WorkItemTemplateField(str, Enum):
    """Enumeration of work item template fields that can be projected in query results."""

    ID = "ID"
    PRODUCT_FAMILIES = "PRODUCT_FAMILIES"
    PART_NUMBERS = "PART_NUMBERS"
    NAME = "NAME"
    SUMMARY = "SUMMARY"
    DESCRIPTION = "DESCRIPTION"
    TEMPLATE_GROUP = "TEMPLATE_GROUP"
    TEST_PROGRAM = "TEST_PROGRAM"
    TYPE = "TYPE"
    TIMELINE = "TIMELINE"
    TIMELINE_ESTIMATED_DURATION_IN_SECONDS = "TIMELINE_ESTIMATED_DURATION_IN_SECONDS"
    RESOURCES = "RESOURCES"
    RESOURCES_SYSTEMS = "RESOURCES_SYSTEMS"
    RESOURCES_ASSETS = "RESOURCES_ASSETS"
    RESOURCES_DUTS = "RESOURCES_DUTS"
    RESOURCES_FIXTURES = "RESOURCES_FIXTURES"
    RESOURCES_ASSETS_FILTER = "RESOURCES_ASSETS_FILTER"
    RESOURCES_DUTS_FILTER = "RESOURCES_DUTS_FILTER"
    RESOURCES_FIXTURES_FILTER = "RESOURCES_FIXTURES_FILTER"
    RESOURCES_SYSTEMS_FILTER = "RESOURCES_SYSTEMS_FILTER"
    WORKSPACE = "WORKSPACE"
    CREATED_BY = "CREATED_BY"
    UPDATED_BY = "UPDATED_BY"
    EXECUTION_ACTIONS = "EXECUTION_ACTIONS"
    FILE_IDS = "FILE_IDS"
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"
    PROPERTIES = "PROPERTIES"
    DASHBOARD = "DASHBOARD"
    WORKFLOW_ID = "WORKFLOW_ID"


class WorkItemTemplateOrderBy(str, Enum):
    """Enumeration of fields by which work item templates can be ordered."""

    ID = "ID"
    NAME = "NAME"
    TEMPLATE_GROUP = "TEMPLATE_GROUP"
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"


class QueryWorkItemTemplatesRequest(JsonModel):
    """Represents the request body content for querying work item templates.
    Allows filtering, sorting, and pagination of work item template results.
    """

    filter: str | None = None
    """A string expression to filter the work item templates returned by the query.

    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable.
    """

    substitutions: List[bool | int | str | None] | None = None
    """Makes substitutions in the query filter expression.

    Substitutions for the query expression are indicated by non-negative integers that are
    prefixed with the @ symbol. Each substitution in the given expression will be replaced by
    the element at the corresponding index (zero-based) in this list.
    """

    take: int | None = None
    """The maximum number of work item templates to return in the response."""

    order_by: WorkItemTemplateOrderBy | None = None
    """The field name to use for sorting the work item templates."""

    descending: bool | None = None
    """Whether to return the work item templates in descending order."""

    continuation_token: str | None = None
    """A token which allows the user to resume a query at the next item in the matching results.

    When querying, a token will be returned if a query may be continued. To obtain the next page
    of results, pass the token to the service on a subsequent request.
    """

    projection: List[WorkItemTemplateField] | None = None
    """
    Gets or sets the projection to be used when retrieving the work item templates. If not specified,
    all properties will be returned.
    """



from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._update_work_item_template_request import UpdateWorkItemTemplateRequest


class UpdateWorkItemTemplatesRequest(JsonModel):
    """Represents the request body content for updating multiple work item templates."""

    work_item_templates: List[UpdateWorkItemTemplateRequest]
    """List of work item templates to update."""

    replace: bool | None = None
    """When true, existing key-value pair fields are replaced instead of merged."""



from typing import List

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._create_work_item_template_request import CreateWorkItemTemplateRequest
from ._work_item_template import WorkItemTemplate


class CreateWorkItemTemplatesPartialSuccessResponse(JsonModel):
    """Response for creating work item templates with partial success."""

    created_work_item_templates: List[WorkItemTemplate]
    """List of successfully created work item templates."""

    failed_work_item_templates: List[CreateWorkItemTemplateRequest] | None = None
    """List of work item template requests that failed during creation."""

    error: ApiError | None = None
    """The error that occurred when creating the work item templates."""
